(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)
(*  Soubor: CAS.PAS                                                        *)
(*  Obsah: stopky (presnost na sekundy)                                    *)
(*         vylepsene procedury pro cekani (presnost na 3 ms)               *)
(*  Autor: Mircosoft                                                       *)
(*         M. Palms: timer                                                 *)
(*         M. Milda: prace s prerusenim, zrychleni casu apod.              *)
(*  Posledni uprava: 2.2.2005                                              *)
(*  Pro kompilaci: (DOS)                                                   *)
(*  Pro spusteni: nic                                                      *)
(*  Upozorneni: tyto zdrojove kody pouzivate na vlastni nebezpeci          *)
(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)
unit Cas;
{$R-}
interface

type
Timer = object{jednoduche a spolehlive stopky}
        stopped:boolean;{stopky zastaveny?}
        konec,zbyva:longint;{systemovy cas prevedeny na sekundy}
        procedure init(time:longint);{spusti stopky (time je v sekundach)}
        function read:longint;{precte zbyvajici cas}
        procedure stop;{zastavi stopky}
        procedure restart;{znovu spusti stopky od mista zastaveni}
        end;{timer}

{vylepsene cekaci procedury:}
procedure StartCekani;
{zacne pocitat cas}
procedure Pockej(doba:word);
{Ceka zadanou dobu od pocatecniho casu. Parametr doba je v ms.
 Vlastne to same jako delay(), ale neceka se od zacatku procedury Pockej,
 ale od zavolani procedury StartCekani. Pokud Startcekani nebyl zavolan, vola
 se na zacatku procedury Pockej automaticky.}
function DosazenCas(doba:word):boolean;
{vraci true v pripade, ze od prikazu startcekani ubehl zadany pocet milisekund}
procedure StopCekani;
{zastavi pocitani casu}
function GetMS:word;
{vrati hodnotu promenne ms}

{Na co je to vsechno vlastne dobre:
begin
 repeat                              Takhle by rychlost provadeni
 dlouha_posloupnost_prikazu;         cyklu zavisela na rychlosti
 delay(60);                          procesoru.
 until nejaka_podminka;
end;

begin
 repeat                              Takhle na ni nezavisi.
 startcekani;
 dlouha_posloupnost_prikazu;
 pockej(60);                          Kdyz je nove preruseni 8
 until nejaka_podminka;               nainstalovano, navenek se nijak
end;                                  neprojevi (vsechno bezi normalne).


Dalsi moznost pouziti:

begin
startcekani;
Nejake_Prikazy;
stopcekani;
writeln('Provedeni Nejakych_Prikazu trvalo presne ',getms,' milisekund.');
end;

Instalace a odinstalace je plne automaticka, staci jednotku pouzit.
}

implementation
uses dos;

const rychlost=20;{pro prijatelnou presnost to staci}

var puvodni8:procedure;{procedura z preruseni 8}
    puvodniExitproc:pointer;{ukazatel na standardni ukoncovaci proceduru}
    counter,{pocita, kdy se ma spustit puvodni preruseni 8
             (kvuli zrychlenemu casu)}
    ms:word;{milisekundy, ktere ubehly od zavolani procedury startcekani.
             Typ word by mel stacit, nepredpokladam, ze by nekdo potreboval
             cekat dele nez minutu.}
    CekaSe:boolean;{pocita se cas?}

function zjisticas:longint;{pro timer}
var ye,mo,da,ho,mi,se,NaNic:word;
Begin
getdate(ye,mo,da,nanic); gettime(ho,mi,se,nanic);
zjisticas:=((((longint(ye-1980)*12+mo)*31+da)*24+ho)*60+mi)*60+se;
End;{zjisticas}

procedure timer.init(time:longint);
Begin
stopped:=false;
zbyva:=time;
konec:=zjisticas+time;
End;{init}

function timer.read:longint;
Begin
if stopped then read:=zbyva
           else read:=konec-zjisticas;
End;{read}

procedure timer.stop;
Begin
if not stopped then zbyva:=konec-zjisticas;
stopped:=true;
End;{stop}

procedure timer.restart;
Begin
if stopped then konec:=zjisticas+zbyva;
stopped:=false;
End;{restart}

procedure speedup(sp:word);{zrychli systemovy cas (nasobi puvodni rychlost hodnotou sp)}
Begin                      {speedup(1) vrati rychlost na puvodni hodnotu}
sp:=65536 div sp;
port[$43]:=$34;            {by M. Milda}
port[$40]:=lo(sp);
port[$40]:=hi(sp);
End;{speedup}

procedure nova8; interrupt;
Begin
if cekase then inc(ms,3);{zkusmo bylo zjisteno, ze rychlosti 20 odpovidaji 3 ms za jeden tik casovace}
dec(counter);
if counter=0 then begin{ubehlo tolik casu, kolik by trval normalni nezrychleny tik casovace}
                  counter:=rychlost;
                  inline($9C);{=pushf, je to potreba, aby puvodni preruseni bezelo spravne}
                  puvodni8;{zavolame puvodni preruseni (stara se o spoustu veci, nemuzeme ho jen tak vyradit)}
                  end;
port[$20]:=$20;{oznameni konce preruseni}
End;{nova8}

procedure startcekani;
Begin
ms:=0; cekase:=true;
End;{startcekani}

procedure pockej(doba:word);
Begin
if not cekase then startcekani;
repeat until ms>=doba;{o promennou ms se stara preruseni, ktere bezi v pozadi}
cekase:=false;
End;{pockej}

function DosazenCas(doba:word):boolean;
Begin
if cekase then dosazencas:=(ms>=doba)
          else dosazencas:=true;{nestojim o nekonecne smycky, ktere by se mohly objevit}
End;{dosazencas}

procedure StopCekani;
Begin
cekase:=false;
End;{stopcekani}

function GetMS:word;
Begin
getms:=ms;
End;{getms}

procedure novaExitproc; far; {tohle se automaticky zavola pri skonceni programu (Exitproc - viz help)}
Begin
exitproc:=puvodniexitproc;{do ukazatele na standardni ukoncovaci proceduru vratime puvodni adresu,
                           kterou jsme si na zacatku programu ulozili}
speedup(1);{vraceni rychlosti casovace na puvodni hodnotu}
setintvec(8,@puvodni8);{vraceni puvodni procedury preruseni}
End;{novaexitproc}

BEGIN
counter:=rychlost; cekase:=false;{inicializace promennych}
puvodniExitproc:=exitproc;{ulozeni standardni ukoncovaci procedury}
exitproc:=@novaExitproc;{vlozeni vlastni ukoncovaci procedury}
getintvec(8,@puvodni8);{ulozeni puvodni procedury z preruseni 8}
setintvec(8,@nova8);{nastaveni nove procedury pro preruseni}
speedup(rychlost);{zrychleni systemoveho casu (normalne totiz tika po 55 ms)}
END.