(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)
(*  Soubor: GRAFIKA.PAS                                                    *)
(*  Obsah: typy souradnice, sprite, obrsoubor                              *)
(*         material pro psani vlastnimi rastrovymi fonty (8x8)             *)
(*         ruzne procedury pro graficky rezim                              *)
(*  Autor: Mircosoft                                                       *)
(*         M. Visniar (ScrOn a ScrOff)                                     *)
(*         Petr Sulla (rychly getimage a putimage)                         *)
(*  Posledni uprava: 12.2.2005                                             *)
(*  Pro kompilaci: GRAPH.TPU, GLOBAL.PAS/TPU                               *)
(*  Pro spusteni: nic                                                      *)
(*  Upozorneni: tyto zdrojove kody pouzivate na vlastni nebezpeci          *)
(* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *)
unit grafika;
{$I-,G+,B-,X-}
interface

type
souradnice = object{tohle vyuziva spousta jinych objektu}
             x,y:integer;
             procedure init(ix,iy:integer);
             end;

{UPOZORNENI: Pokud neni uvedeno jinak, vsechny promenne oznacene sirka a vyska
jsou o 1 mensi nez sirka a vyska daneho obrazku v pixelech. Duvod je prosty,
viz priklad: getmem(p,imagesize(0,0,sirka,vyska));
getimage(x,y,x+sirka,y+vyska); atd., zkratka lepe se s tim pocita}

sprite = object{pro zjednoduseni nacitani a zobrazovani obrazku z obrazovky}
         sprp:pointer;{ukazatel na misto v pameti (obrazek)}
         sirka,vyska:word;
         function init(is,iv:word):boolean;{getmem, funkce vraci true, pokud se operace zdarila}
         procedure nacti(nx,ny:integer);{kdyz obrazek zasahuje mimo obrazovku,automaticky se posune tak,aby se dal cely nacist}
         procedure zobraz(zx,zy:integer);{- '' -}
         procedure zrus;{freemem}
         end;{sprite}

{obrazky ukladam do souboru formatu *.OBR . Struktura souboru vypada takto:
sirka (word), vyska (word), pocet(byte), obrazek 1, obrazek 2 .. obrazek n.
Tj. do jednoho souboru se da ukladat i vic obrazku. Typ obrsoubor nacte
vsechny obrazky z takoveho souboru do jednosmerne kruhove dynamicke struktury
a muze je pak ruzne zobrazovat.}
uop = ^obrprvek;
obrprvek = record
           p:pointer;
           dalsi:uop;
           end;
obrsoubor = object
            sirka,vyska:word;{vsechny obrazky jsou stejne velke}
            pocet:byte;{pocet obrazku v souboru}
            prvni,{ukazatel na prvni obrazek}
            vybrany:uop;{ukazatel na aktualni obrazek}
            procedure init(isoubor:string;var chyba:byte);{nacteni ze souboru,
     Chyba: 0 = OK, 1 = soubor neexistuje nebo nejde otevrit, 2 = malo pameti}
            procedure zobraz(zx,zy:integer);{zobrazeni aktualniho obrazku, zx,zy je levy horni roh}
            procedure ojednodal;{posunuti ukazatele vybrany o 1 obrazek dal,
                                 z posledniho pak skoci zpet na prvni}
            procedure zrus;{vyklidi pamet}
            end;{obrsoubor}

font8x8=array[#0..#255,0..7]of byte;
{Znakova sada pro pismo o rozmerech 8x8 pixelu. Kazde pismeno je tvoreno
polem osmi bytu, kde kazde cislo (v binarnim tvaru) predstavuje jeden radek
pixelu. Kde je jednicka, tam se pixel vykresli, kde je nula, tam ne.}

function NactiFont8x8(var fn:font8x8;soubor:string):boolean;
{nacte ze souboru font 8x8 pixelu, vraci true, pokud se to povede. Pro tyto
soubory pouzivam koncovku *.F88 (jmeno piste i s ni).}
procedure Print8x8(ix,iy:integer;var FN:font8x8;Barva:byte;Textik:string);
{napise na souradnice ix,iy retezec Textik fontem FN a barvou Barva,
je cca 1.4 krat pomalejsi nez Outtextxy. FN je Var kvuli rychlosti}
procedure Print8x8v(ix,iy:integer;var fn:font8x8;vzorek,barva,velikost:byte;textik:string);
{totez, ale muzete zadat i velikost textu a dokonce i styl vyplne (hodnoty
jako pro setfillstyle). Je cca 1.9 krat pomalejsi nez Outtextxy.
Print8x8 i Print8x8v muzete zadat i zaporne souradnice - vykresli se vzdy,
i kdyz jakakoli cast textu zasahuje mimo obrazovku (to Outtextxy neumi :-) ).}
procedure print8x8s(ix,iy:integer;var fn:font8x8;barva,sirkapismen:byte;textik:string);
{to same jako print8x8, ale da se zadat roztec mezi znaky. Parametr
sirkapismen urcuje vodorovnou vzdalenost levych hornich rohu dvou sousednich
znaku v pixelech. Vhodne, pokud si vytvorite nejaky mensi font.}
function UlozFont8x8(var fn:font8x8;soubor:string):boolean;
{ulozi font 8x8 do souboru, vraci true, pokud se to povede. Pro pouziti
v nejakem programu, ve kterem se ten font nejdriv vytvori.}
procedure GrabFont8x8(var fn:font8x8);
{nacte znakovou sadu tak, ze postupne vykresli na obrazovku vsechny znaky
aktualniho fontu (Outtextxy) a nacita je natvrdo Getpixelem. Jen pro extremni
pripady (nemate zadny *.F88 soubor a nutne potrebujete, aby program bezel
a podobne).}
procedure napis(x,y:integer;bt,bp:byte;hlaska:string);
{Nakresli obdelnik o sirce a vysce textu a text do nej napise. Libovolny font,
zarovnani pouze vychozi - settextjustify(lefttext,toptext). Vhodne pro
prepisovani textu pres sebe.
 x,y - souradnice textu (levy horni roh), obdelnik pod textem zacina kousek
       pred timto bodem
 bt - barva textu
 bp - barva obdelniku pod textem
 hlaska - text k napsani}
procedure napis2(x,y:integer;fn:font8x8;bt,bp:byte;hlaska:string);
{to same, ale misto outtextxy se pise procedurou print8x8}
procedure stintext(x,y:integer;bt,bp:byte;delta:shortint;textik:string);
{stinovany text.
 bt - barva textu    bp - barva stinu
 delta - vzdalenost stinu (ve smeru doprava a dolu) od textu, muze byt i
 zaporna, v takovem pripade muzete stin udelat svetlejsi nez text a pismo pak
 vypada plasticky}
procedure kreslivobr(st:string;ussx,ussy:integer);
{Vykresli obrazek ze souboru st (*.MAC) na souradnicich ussx,ussy.
Tyto obrazky se vytvareji mym editorem AlmostCAD. Jde o soubor grafickych
prvku - cara, obdelnik, elipsa, text, atd. Vyhoda je, ze tyto soubory
nezabiraji moc mista, podle detailnosti vykresleni max. 10 kB, ale prumerne
tak 4 kB (nezalezi na vizualni velikosti, jen na poctu nakreslenych prvku).}
procedure kreslirobr(st:string;ussx,ussy:integer);{nevyzkouseno}
{Vykresli obrazek ze souboru typu *.OBR.
ussx,ussy - levy horni roh. Neprobiha zadna kontrola, jestli je obrazek cely
            na obrazovce.
st - jmeno souboru, piste i s koncovkou
V pripade nejake chyby se misto vykresleni obrazku vypise chybove hlaseni.}
function Adapter:string;
{Vraci jmeno graficke karty. Celkem k nicemu.}
procedure ScrOff;
{vypne obrazovku, tj.: obrazovka cela zcerna, ale vsechny graficke operace,
ktere provedete, budou po opetovnem zapnuti videt. Pri vypnute obrazovce je
zapis do VRAM o neco rychlejsi.}
procedure ScrOn;
{opet zapne obrazovku}
procedure SaveScr(Cislo:byte);
{Ulozi kopii obrazovky do souboru SCREENxx.$$$, kde xx je Cislo.
Format souboru: nicim jinym nez procedurou Loadscr ho neotevrete.}
procedure LoadScr(Cislo:byte;SmazatSoubor:boolean);
{Nacte a zobrazi ulozenou kopii obrazovky (soubor SCREENxx.$$$). Parametr
Smazatsoubor urcuje, jestli se ma soubor po skonceni procedury vymazat.}
procedure WaitRetrace;
{ceka na navrat vykreslovaciho paprsku na obrazovce. Odstrani blikani obrazu
pri animaci, staci ji pouzit pred zobrazenim obrazku. Funguje urcite v rezimu
13h, jinak nevim.}
function TextHeight256(s:string):word;
{v originalni procedure textheight je chyba: kdyz pracujete s grafickym
ovladacem SVGA256.BGI (mozna i s jinymi, to jsem nezkousel, ale EGAVGA je OK)
a mate nastaveny font 0 (defaultfont), vraci stale 8 pixelu bez ohledu na
velikost pisma. Tahle funkce to resi, pouziti je uplne stejne a je plne
kompatibilni}
function fImageSize(x,y,x0,y0:word):word;
{nutna pro nasledujici dve procedury, protoze struktura obrazku je trochu jina
nez bezna borlandovska}
procedure fGetImage(x,y,x0,y0:word;p:pointer);
{superrychly getimage a...}
procedure fPutImage(x,y:word;p:pointer);
{...taktez superrychly putimage. X-ova souradnice musi byt nasobkem osmi,
zaokrouhluje se automaticky. Oboje je pouzitelne jen v gr. rezimu 640x480x16
(standardni rezim s pouzitim EgaVga.bgi). Nepodporuje xorput, ale co taky
s nim. Pozor, ze se obrazek nevola jako p^, ale jen p.}
procedure KresliVStrafek(x,y:integer;sirka,vyska,cislo:word;vl,bl,vp,bp:byte);
{x,y - souradnice leveho horniho rohu
 sirka, vyska - skutecna sirka a vyska v pixelech (neni o 1 mensi)
 cislo - skutecna sirka leve casti obdelniku
 vl,bl - vypln a barva leve casti obdelniku
 vp,bp - vypln a barva prave casti obdelniku
Vhodne napr. do her jako ukazatel poctu zivotu apod.}
procedure KresliSStrafek(x,y:integer;sirka,vyska,cislo:word;vh,bh,vd,bd:byte);
{totez, ale svisle; cislo je skutecna vyska spodni casti obdelnicku}
procedure KVSoddo(x,y:integer;sirka,vyska,cOd,cDo:word;vl,bl,vp,bp:byte);
procedure KSSoddo(x,y:integer;sirka,vyska,cOd,cDo:word;vh,bh,vd,bd:byte);
{funkce a parametry stejne, ale kresli se jen rozdil mezi polohou cOd k nove
poloze cDo. Kvuli rychlosti a odstraneni blikani}
procedure StinCara(x1,y1,x2,y2:integer);
{stinovana cara. Barva cary je takova, jakou si nastavite Setcolorem (zustane
zachovana) a stin je cerny.}
procedure PushScr16;
{nacte obrazovku a ulozi ji v pameti. Jen pro 640x480x16. Da se ulozit i vic
obrazovek po sobe (zasobnik).}
procedure LodScr16;
{zobrazi posledne ulozenou obrazovku}
procedure PopScr16;
{zobrazi posledne ulozenou obrazovku a odstrani ji z pameti}

implementation
uses graph,global;

procedure souradnice.init(ix,iy:integer);
Begin
x:=ix; y:=iy;
End;{init}

function sprite.init(is,iv:word):boolean;
var velikost:word;
    chyba:boolean;
Begin
sirka:=is; vyska:=iv;
velikost:=imagesize(0,0,sirka,vyska);
chyba:=(velikost>maxavail){vetsi nez nejvetsi souvisle volne misto v pameti}
       or(velikost=0);{vetsi nez cela pamet}
if chyba then sprp:=nil
         else begin
              getmem(sprp,velikost);
              nacti(0,0);{pojistka}
              end;
init:=not chyba;
End;{init}

procedure sprite.nacti(nx,ny:integer);
var mx,my:word;
Begin
if sprp<>nil then
 begin
 mx:=getmaxx; my:=getmaxy;
 if nx<0 then nx:=0; if ny<0 then ny:=0;{pokud by nacitana oblast zasahovala}
 if nx+sirka>mx then nx:=mx-sirka;     {mimo obrazovku, posune se tak, aby}
 if ny+vyska>my then ny:=my-vyska;    {byla na obrazovce cela. velmi uzitecne}
 getimage(nx,ny,nx+sirka,ny+vyska,sprp^);
 end;
End;{nacti}

procedure sprite.zobraz(zx,zy:integer);
var mx,my:word;
Begin
if sprp<>nil then
 begin
 mx:=getmaxx; my:=getmaxy;
 if zx<0 then zx:=0; if zy<0 then zy:=0;
 if zx+sirka>mx then zx:=mx-sirka;
 if zy+vyska>my then zy:=my-vyska;
 putimage(zx,zy,sprp^,0);
 end;
End;{zobraz}

procedure sprite.zrus;
Begin
if sprp<>nil then
 begin
 freemem(sprp,imagesize(0,0,sirka,vyska));
 sprp:=nil;
 end;
End;{zrus}

procedure obrsoubor.init(isoubor:string;var chyba:byte);
var f:file;
    velikost:word;
    i:byte;
Begin
chyba:=0;
assign(f,isoubor);
reset(f,1);
if ioresult<>0 then begin chyba:=1; exit; end;{soubor neni nebo nejde otevrit}
blockread(f,sirka,2);
blockread(f,vyska,2);
blockread(f,pocet,1);
velikost:=imagesize(0,0,sirka,vyska);
if(velikost>maxavail)or(velikost=0)or(pocet*velikost>memavail)then begin chyba:=2; exit; end;{nevejde se do pameti}
new(prvni);
vybrany:=prvni;
for i:=1 to pocet do begin
                     getmem(vybrany^.p,velikost);
                     blockread(f,vybrany^.p^,velikost);
                     if i<pocet then begin
                                     new(vybrany^.dalsi);
                                     ojednodal;
                                     end
                                else vybrany^.dalsi:=prvni;
                     end;
close(f);
vybrany:=prvni;
End;{init}

procedure obrsoubor.zobraz(zx,zy:integer);
Begin
putimage(zx,zy,vybrany^.p^,0);
End;{zobraz}

procedure obrsoubor.ojednodal;
Begin
vybrany:=vybrany^.dalsi;
End;{ojednodal}

procedure obrsoubor.zrus;
var velikost:word;
    i:byte;
Begin
velikost:=imagesize(0,0,sirka,vyska);
vybrany:=prvni;
for i:=1 to pocet do begin
                     freemem(vybrany^.p,velikost);
                     ojednodal;
                     dispose(prvni);
                     prvni:=vybrany;
                     end;
prvni:=nil; vybrany:=nil;
End;{zrus}

function nactifont8x8(var fn:font8x8;soubor:string):boolean;
var f:file of font8x8;
Begin                        { kdybych netestoval uspesnost I/O operaci,    }
assign(f,soubor);            { zkratila by se cela procedura na 4 radky :-) }
reset(f);
if ioresult=0 then begin
                   read(f,fn);
                   if ioresult=0 then begin
                                      close(f);
                                      nactifont8x8:=true;
                                      end
                                 else nactifont8x8:=false;
                   end
              else nactifont8x8:=false;
End;{nactifont8x8}

procedure print8x8(ix,iy:integer;var fn:font8x8;barva:byte;textik:string);
var x,y,index:byte;
Begin
dec(ix,8);{aby se s tim lip pocitalo - prvni znak retezce ma totiz index 1}
for index:=1 to byte(textik[0]) do
  begin
  for y:=0 to 7 do
   for x:=0 to 7 do
     if ((fn[textik[index],y] shr x)and 1)=1 then putpixel(ix+index shl 3+x,iy+y,barva);
  end;
End;{print8x8}

procedure print8x8v(ix,iy:integer;var fn:font8x8;vzorek,barva,velikost:byte;textik:string);
var x,y,index,pv:byte;
Begin
if velikost=0 then velikost:=1;{blbuvzdornost predevsim}
dec(ix,8*velikost);
setfillstyle(vzorek,barva);
pv:=pred(velikost);{aby se nemusela odcitat 1 pri kazdem cyklu}
for index:=0 to byte(textik[0]) do
  begin
  for y:=0 to 7 do
   for x:=0 to 7 do if ((fn[textik[succ(index)],y] shr x)and 1)=1 then
        bar(ix+(index shl 3)*velikost+x*velikost,iy+y*velikost,ix+(index shl 3)*velikost+x*velikost+pv,iy+y*velikost+pv);
  end;
End;{print8x8v}

procedure print8x8s(ix,iy:integer;var fn:font8x8;barva,sirkapismen:byte;textik:string);
var x,y,index:byte;
Begin
dec(ix,sirkapismen);
for index:=1 to byte(textik[0]) do
  begin
  for y:=0 to 7 do
   for x:=0 to 7 do
     if ((fn[textik[index],y] shr x)and 1)=1 then putpixel(ix+index*sirkapismen+x,iy+y,barva);
  end;
End;{print8x8s}

function ulozfont8x8(var fn:font8x8;soubor:string):boolean;
var f:file of font8x8;
Begin
assign(f,soubor);
rewrite(f);
if ioresult=0 then begin
                   write(f,fn);
                   if ioresult=0 then begin
                                      close(f);
                                      if ioresult=0 then ulozfont8x8:=true
                                                    else ulozfont8x8:=false;
                                      end
                                 else ulozfont8x8:=false;
                   end
              else ulozfont8x8:=false;
End;{ulozfont8x8}

procedure grabfont8x8(var fn:font8x8);
var i:char;
    x,y:byte;
    pole:array[0..7,0..7] of byte;
    pom:byte;
Begin
setcolor(1); setfillstyle(1,0);
for i:=#0 to #255 do begin
                     bar(0,0,7,7);{vymazani minuleho pismena}
                     outtextxy(0,0,i);{zobrazeni noveho pismena}
                     for x:=0 to 7 do             {nacteni pismena do pole}
                      for y:=0 to 7 do pole[x,y]:=getpixel(x,y);
                     pom:=0;
                     for y:=0 to 7 do
                      begin  {zakodovani pismena z pole do datove struktury fontu}
                      for x:=7 downto 0 do begin
                                           pom:=pom shl 1;
                                           pom:=pom or pole[x,y];
                                           end;
                      fn[i,y]:=pom;
                      end;
                     end;
End;{grabfont8x8}

procedure napis(x,y:integer;bt,bp:byte;hlaska:string);
Begin
setcolor(bt); setfillstyle(1,bp);
bar(x-2,y-1,x+textwidth(hlaska),y+textheight(hlaska));{pozadi}
outtextxy(x,y,hlaska);{text}
End;{napis}

procedure napis2(x,y:integer;fn:font8x8;bt,bp:byte;hlaska:string);
Begin
setfillstyle(1,bp); bar(x-2,y-1,x+byte(hlaska[0])shl 3,y+8);{pozadi}
print8x8(x,y,fn,bt,hlaska);{text}
End;{napis}

procedure stintext(x,y:integer;bt,bp:byte;delta:shortint;textik:string);
Begin
setcolor(bp);
outtextxy(x+delta,y+delta,textik);{stin}
setcolor(bt);
outtextxy(x,y,textik);{text}
End;{stintext}

procedure kreslivobr(st:string;ussx,ussy:integer);
const
pbod=1; pcara=2; prectangle=3; pcircle=4; pelipsa=5; poblouk=6; pbar=7; pctyruh=8; pfelipsa=9; ptext=10;
var
x,y,x2,y2,x3,y3,x4,y4,ix,iy:integer;
cotoje,barva,vypln,stylcary,tloustka,b:byte;
polomer,polomer2,stuhel,koncuhel,w:word;
formattextu,ti:textsettingstype;
ret:string[50];
f:file;
ctyruh:array[1..4] of pointtype;
Begin
assign(f,st); reset(f,1);
if ioresult=0 then
while not eof(f)do begin
                   {nacitani ze souboru:}
                   blockread(f,x,2);
                   blockread(f,y,2);
                   blockread(f,b,1);
                   barva:=b and 15; vypln:=b shr 4;{2 cisla ulozena v 1 bytu}
                   blockread(f,cotoje,1);
                   if cotoje in [pcara,prectangle,pbar,pctyruh] then begin
                                                                     blockread(f,x2,2);
                                                                     blockread(f,y2,2);
                                                                     end;
                   if cotoje=pctyruh then begin
                                          blockread(f,x3,2);
                                          blockread(f,y3,2);
                                          blockread(f,x4,2);
                                          blockread(f,y4,2);
                                          end;
                   if cotoje in [pcircle,pelipsa,pfelipsa,poblouk] then blockread(f,polomer,2);
                   if cotoje in [pelipsa,pfelipsa] then blockread(f,polomer2,2);
                   if cotoje=poblouk then begin
                                          blockread(f,stuhel,2);
                                          blockread(f,koncuhel,2);
                                          end;
                   if cotoje=ptext then begin
                                        blockread(f,w,2);
                                        with formattextu do
                                          begin
                                          font:=w and 15;
                                          direction:=(w shr 4)and 1;
                                          horiz:=(w shr 5)and 3;
                                          vert:=(w shr 7)and 3;
                                          charsize:=w shr 9;
                                          end;
                                        blockread(f,ret[0],1);
                                        for b:=1 to byte(ret[0]) do blockread(f,ret[b],1);
                                        end;
                   if cotoje in [pcara,prectangle,pcircle,poblouk,pelipsa,pfelipsa,pctyruh] then
                                           begin
                                           blockread(f,b,1);
                                           stylcary:=b and 15; tloustka:=b shr 4;
                                           end;
                   {kresleni:}
                   setcolor(barva); setfillstyle(vypln,barva);
                   setlinestyle(stylcary,$F18F,tloustka);
                   ix:=ussx+x; iy:=ussy+y;
                   case cotoje of pbod:putpixel(ix,iy,barva);
                                  pcara:line(ix,iy,ix+x2,iy+y2);
                                  prectangle:rectangle(ix,iy,ix+x2,iy+y2);
                                  pcircle:circle(ix,iy,polomer);
                                  pelipsa:ellipse(ix,iy,0,360,polomer,polomer2);
                                  poblouk:arc(ix,iy,stuhel,koncuhel,polomer);
                                  pbar:bar(ix,iy,ix+x2,iy+y2);
                                  pctyruh:begin
                                          ctyruh[1].x:=ix;    ctyruh[1].y:=iy;
                                          ctyruh[2].x:=ix+x2; ctyruh[2].y:=iy+y2;
                                          ctyruh[3].x:=ix+x3; ctyruh[3].y:=iy+y3;
                                          ctyruh[4].x:=ix+x4; ctyruh[4].y:=iy+y4;
                                          fillpoly(4,ctyruh);
                                          end;
                                  pfelipsa:fillellipse(ix,iy,polomer,polomer2);
                                  ptext:begin
                                        gettextsettings(ti);{zaloha puvodniho nastaveni}
                                        with formattextu do begin
                                                            settextstyle(font,direction,charsize);
                                                            settextjustify(horiz,vert);
                                                            end;
                                        outtextxy(ix,iy,ret);
                                        with ti do begin{puvodni nastaveni}
                                                   settextstyle(font,direction,charsize);
                                                   settextjustify(horiz,vert);
                                                   end;
                                        end;
                                  end;
                   setlinestyle(0,0,1);{puvodni nastaveni (jine nepredpokladam)}
                   end;
close(f); if ioresult=0 then ;{pojistka pro pripad, ze by se soubor nepodarilo zavrit}
End;{kreslivobr}

procedure kreslirobr(st:string;ussx,ussy:integer);
var f:file;
    sirka,vyska,velikost:word;
    b:byte;
    p:pointer;
Begin
assign(f,st);
reset(f,1);
if ioresult=0 then
  begin
  blockread(f,sirka,2);
  blockread(f,vyska,2);
  blockread(f,b,1);{pocet - neuplatni se, protoze se stejne kresli jen prvni obrazek}
  velikost:=imagesize(0,0,sirka,vyska);
  if velikost<=maxavail then begin
                             getmem(p,velikost);
                             blockread(f,p^,velikost);
                             close(f);
                             putimage(ussx,ussy,p^,0);
                             freemem(p,velikost);
                             end
                        else outtextxy(ussx,ussy,'Nedostatek pameti pro vykresleni obrazku.');
  end
else outtextxy(ussx,ussy,'Soubor s obrazkem nebyl nalezen.');
End;{kreslirobr}

function Adapter:string;
var kod,pom:byte;
Begin
adapter:='neznamy';
asm
mov AX,$1A00       {AX:=$1A00}
int $10            {intr(16,registry)}
mov pom,AL         {pom:=AL}
mov kod,BL         {kod:=BL}
end;
if pom<>$1A then kod:=0;
case kod of 1:adapter:='Hercules';
            2:adapter:='CGA';
            4,5:adapter:='EGA';
            7,8:adapter:='VGA';
            10..12:adapter:='MCGA';
            end;
End;{adapter}

procedure scron;
Begin
port[$3c4]:=1;                       {by M. Visniar}
port[$3c5]:=port[$3c5] and $df;
End;{scron}

procedure scroff;
Begin
port[$3c4]:=1;                       {by M. Visniar}
port[$3c5]:=port [$3c5] or $20;
End;{scroff}

procedure savescr(cislo:byte);
var f:file;{soubor, do ktereho se to ulozi}
    soubor:string[12];{jmeno toho souboru}
    p:pointer;{buffer}
    i:byte;{pomocna}
    velikost:word;{velikost bufferu v bytech}
Begin
velikost:=imagesize(0,0,639,9);
if cislo>99 then cislo:=99;{cislo max. dvojciferne}
str(cislo,soubor);{prevod cisla na retezec}
if length(soubor)=1 then soubor:='0'+soubor;{pokud je cislo jednociferne, pripise pred nej nulu}
soubor:='SCREEN'+soubor+'.$$$';{slozeni celeho jmena souboru}
if maxavail>=velikost then{je dost volne pameti?}
  begin
  assign(f,soubor);
  rewrite(f,velikost);{otevreni souboru}
  if ioresult=0 then
    begin
    getmem(p,velikost);{alokace potrebne pameti}
    for i:=0 to 47 do begin
                      getimage(0,10*i,639,10*i+9,p^);{nacteni casti obrazovky}
                      blockwrite(f,p^,1);{zapis teto casti do souboru}
                      end;
    close(f);{zavreni souboru}
    if ioresult=0 then ;{pojistka}
    freemem(p,velikost);{uvolneni pameti}
    end;
  end;
End;{savescr}

procedure loadscr(cislo:byte;smazatsoubor:boolean);
var f:file;{soubor, ze ktereho se to ma nacist}
    soubor:string[12];{jmeno toho souboru}
    p:pointer;{buffer}
    i:byte;{pomocna}
    velikost:word;
Begin
velikost:=imagesize(0,0,639,9);
if cislo>99 then cislo:=99;{cislo max. dvojciferne}
str(cislo,soubor);{prevod cisla na retezec}
if length(soubor)=1 then soubor:='0'+soubor;{pokud je cislo jednociferne, pripise pred nej nulu}
soubor:='SCREEN'+soubor+'.$$$';{slozeni celeho jmena}
if maxavail>=velikost then{je dost volne pameti?}
  begin
  assign(f,soubor);
  reset(f,velikost);{otevreni souboru}
  if ioresult=0 then
    begin
    getmem(p,velikost);{alokace pameti}
    for i:=0 to 47 do begin
                      blockread(f,p^,1);{nacteni casti obrazovky ze souboru}
                      putimage(0,10*i,p^,0);{vykresleni teto casti na obrazovku}
                      end;
    close(f);{zavreni souboru}
    freemem(p,velikost);{uvolneni pameti}
    if smazatsoubor then erase(f);{pripadne vymazani souboru}
    if ioresult=0 then ;{pojistka}
    end;
  end;
End;{loadscr}

procedure WaitRetrace; assembler;
Asm
    MOV     dx,3DAh
@l1:
    IN      al,dx
    AND     al,01h
    JNZ     @l1
@l2:
    IN      al,dx
    AND     al,01h
    JZ      @l2
End;{waitretrace}

function TextHeight256(s:string):word;
var ti:textsettingstype;
Begin
gettextsettings(ti);
if ti.font=0 then textheight256:=ti.charsize shl 3{= krat 8. Font 0 je vzdy 8 bodu vysoky}
             else textheight256:=textheight(s);
End;{textheight256}

function fimagesize(x,y,x0,y0:word):word; {by Petr Sulla (1996)}
var dummy:longint;
    sx,sy:word;
Begin
sx:=succ(abs(x0-x) shr 3);
sy:=succ(abs(y0-y));
dummy:=((sx*sy)shl 2)+10;
if(dummy<65520)and(global_grafrezim=(gdvga shl 8)or gm640x480x16)
      then fimagesize:=dummy
      else fimagesize:=0;
End;{fimagesize}

procedure fgetimage(x,y,x0,y0:word;p:pointer);assembler; {by Petr Sulla (1996)}
var m:word;
Asm
    push es
    push ds
    cld                    {rep adds}
    les di,[p]             {x0 <- width, y0 <- height, write to buffer}
    mov ax,x0
    sub ax,x
    shr ax,3               {x0:=x0 div 8}
    mov x0,ax
    mov es:[di],ax
    inc x0
    mov ax,y0
    sub ax,y
    mov y0,ax
    mov es:[di+2],ax
    inc y0
    add di,4               {es:di <- pointer to data}
    shr x,3                {x:=x div 8}
    mov bx,y               {m:=y*80+x - offset in vram}
    mov cx,bx
    shl bx,4
    shl cx,6
    add bx,cx
    add bx,x
    mov m,bx
    mov ax,0a000h          {ds:si <- beginning of vram}
    mov ds,ax
    mov bx,y0              {bx - lines counter}
    mov dx,03ceh           {dx - port address}
    mov al,4
    out dx,al
    inc dx
@@1:
    mov ah,4               {ah - 3=1.bit plane, 2=2.bitpl.,1=3.bitpl.,0=4.bitpl.}
@@2:
    mov al,ah
    dec al
    out dx,al              {send number of bitplane to the graphic card}
    mov si,m               {offset in videoram}
    mov cx,x0              {image width to counter}
    rep movsb              {send  cx bytes from DS:SI(vram) to ES:DI(image)}
    dec ah                 {decrement al - next bitplane}
    jnz @@2                {is zero ? - no=next bitplane}
    add m,80               {next line in vram}
    dec bx                 {decrement lines counter}
    jnz @@1                {last line  ? no=next line}
    dec dx                 {set graphic card back to std. modus}
    mov al,3
    out dx,al
    inc dx
    xor al,al
    out dx,al
    pop ds
    pop es
End;{fgetimage}

procedure fputimage(x,y:word;p:pointer);assembler;    {by Petr Sulla (1996)}
var sx,sy,m:word;
Asm
    push es
    push ds
    cld                    {rep adds}
    shr x,3                {x:=x div 8}
    mov bx,y               {m:=y*80+x - offset in vram}
    mov cx,bx
    shl bx,4
    shl cx,6
    add bx,cx
    add bx,x
    mov m,bx
    lds si,[p]             {sx <- width, sy <- height}
    mov ax,word ptr ds:[si]
    inc ax
    mov sx,ax
    mov ax,word ptr ds:[si+2]
    inc ax
    mov sy,ax
    add si,4               {ds:si <- pointer to data}
    mov ax,0a000h          {es:di <- beginning of vram}
    mov es,ax
    mov bx,sy              {bx - lines counter}
    mov dx,03c4h           {dx - port address}
    mov al,2
    out dx,al
    inc dx
@@1:
    mov al,8               {al - 8=1.bit plane, 4=2.bitpl.,2=3.bpl.,1=4.bpl.}
@@2:
    out dx,al              {send bitplane number to the graphic card}
    mov di,m               {offset in videoram}
    mov cx,sx              {image width to counter}
    rep movsb              {send cx bytes from DS:SI(image) to ES:DI(vram)}
    shr al,1               {divide al by 2 - next bitplane of 4}
    jnz @@2                {is zero ? - no=next bitplane}
    add m,80               {next line in vram}
    dec bx                 {decrement lines counter}
    jnz @@1                {last line ? no=next line}
    dec dx                 {set graphic card back to std. modus}
    mov al,2
    out dx,al
    inc dx
    mov al,15
    out dx,al
    pop ds
    pop es
End;{fputimage}

procedure KresliVStrafek(x,y:integer;sirka,vyska,cislo:word;vl,bl,vp,bp:byte);
Begin
if cislo>sirka then cislo:=sirka;
dec(vyska);
if cislo<sirka then begin
                    setfillstyle(vp,bp);
                    bar(x+cislo,y,x+sirka-1,y+vyska);
                    end;
if cislo>0 then begin
                setfillstyle(vl,bl);
                bar(x,y,x+cislo-1,y+vyska);
                end;
End;{kreslivstrafek}

procedure KresliSStrafek(x,y:integer;sirka,vyska,cislo:word;vh,bh,vd,bd:byte);
Begin
if cislo>vyska then cislo:=vyska;
dec(sirka);
if cislo<vyska then begin
                    setfillstyle(vh,bh);
                    bar(x,y,x+sirka,y+vyska-cislo-1);
                    end;
if cislo>0 then begin
                setfillstyle(vd,bd);
                bar(x,y+vyska-cislo,x+sirka,y+vyska-1);
                end;
End;{kreslisstrafek}

procedure KVSoddo(x,y:integer;sirka,vyska,cod,cdo:word;vl,bl,vp,bp:byte);
Begin
if cod>sirka then cod:=sirka;
if cdo>sirka then cdo:=sirka;
if cod>cdo then begin
                setfillstyle(vp,bp);
                bar(x+cod-1,y,x+cdo-1,y+vyska-1);
                end
           else if cod<cdo then begin
                                setfillstyle(vl,bl);
                                bar(x+cod-1,y,x+cdo-1,y+vyska-1);
                                end;
                          {else nic}
End;{kvsoddo}

procedure KSSoddo(x,y:integer;sirka,vyska,cod,cdo:word;vh,bh,vd,bd:byte);
Begin
if cod>vyska then cod:=vyska;
if cdo>vyska then cdo:=vyska;
if cod>cdo then begin
                setfillstyle(vh,bh);
                bar(x,y+vyska-cod,x+sirka-1,y+vyska-cdo-1);
                end
           else if cod<cdo then begin
                                setfillstyle(vd,bd);
                                bar(x,y+vyska-cod-1,x+sirka-1,y+vyska-cdo);
                                end;
End;{kssoddo}

procedure StinCara(x1,y1,x2,y2:integer);
var mb:byte;
Begin
mb:=getcolor;{zalohovani nastavene barvy}
setcolor(0); line(succ(x1),succ(y1),succ(x2),succ(y2));{stin}
setcolor(mb); line(x1,y1,x2,y2);{cara}
End;{stincara}

type
uknaobr=^obrazovka16;
obrazovka16 = record
              p1,p2,p3:pointer;
              dalsi:uknaobr;
              end;

var zasobnikobr:uknaobr;

procedure pushscr16;
var velikost:word;
    p:uknaobr;
Begin
velikost:=fimagesize(0,0,639,159);
if (velikost>0)and(3*velikost<maxavail)then
 begin
 new(p);
 if zasobnikobr=nil then begin
                         zasobnikobr:=p;
                         p^.dalsi:=nil;
                         end
                    else begin
                         p^.dalsi:=zasobnikobr;
                         zasobnikobr:=p;
                         end;
 with p^ do begin
            getmem(p1,velikost); getmem(p2,velikost); getmem(p3,velikost);
            fgetimage(0,0,639,159,p1);
            fgetimage(0,160,639,319,p2);
            fgetimage(0,320,639,479,p3);
            end;
 end;
End;{pushscr16}

procedure lodscr16;
Begin
with zasobnikobr^ do begin
                     fputimage(0,0,p1);
                     fputimage(0,160,p2);
                     fputimage(0,320,p3);
                     end;
End;{lodscr16}

procedure popscr16;
var velikost:word;
    p:uknaobr;
Begin
velikost:=fimagesize(0,0,639,159);
with zasobnikobr^ do begin
                     fputimage(0,0,p1);
                     fputimage(0,160,p2);
                     fputimage(0,320,p3);
                     freemem(p1,velikost); freemem(p2,velikost); freemem(p3,velikost);
                     end;
p:=zasobnikobr;
zasobnikobr:=zasobnikobr^.dalsi;
dispose(p);
End;{popscr16}

END.